(*
 *	MPEG Audio Encoder for DirectShow
 *
 *	Copyright (c) 2000 Marie Orlova, Peter Gubanov, Elecard Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Delphi Translation (C) 2003 Milenko Mitrovic <dcoder@dsp-worx.de>
 *
 * changes:
 * --------
 * (Jun 2009 by Martin Offenwanger <coder@dsplayer.de> - www.dsplayer.de )
 *  - updated Interface (LAME v3.98.x)
 *  - added supported values and dependency comments to all Interface methods
 *)

unit LameDShowIntf;

interface

uses
  Windows, ActiveX, DirectShow9;

const
  IID_IAudioEncoderProperties: TGUID = '{ca7e9ef0-1cbe-11d3-8d29-00a0c94bbfee}';

 //////////////////////////////////////////////////////////////////////////////
 // Configuring MPEG audio encoder parameters with unspecified
 // input stream type may lead to misbehaviour and confusing
 // results. In most cases the specified parameters will be
 // overridden by defaults for the input media type.
 // To archive proper results use this interface on the
 // audio encoder filter with input pin connected to the valid
 // source.
 //////////////////////////////////////////////////////////////////////////////

type
  IAudioEncoderProperties = interface(IUnknown)
    ['{ca7e9ef0-1cbe-11d3-8d29-00a0c94bbfee}']

    ////////////////////////////////////////////////////////////////////////////
    // PES Output Enabled/Disabled Switch:
    // "0 = PES disabled"    "1 = PES enabled"
    //-------------------------------------------------------------------------
    // Is PES enabked/disabled state
    function get_PESOutputEnabled(out dwEnabled: DWORD): HRESULT; stdcall;
    // Enable/disable PES output
    function set_PESOutputEnabled(dwEnabled: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Constant Bitrate Vaulue
    //-------------------------------------------------------------------------
    // Get target compression bitrate - Supported Values (Kbits/s):
    // 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 192, 224,
    // 256, 320
    function get_Bitrate(out dwBitrate: DWORD): HRESULT; stdcall;
    // Set target compression bitrate (Kbits/s)
    //........................................................................
    // Setting the correct supported bit rate (Kbits/s) values depends on the
    // selected sample rate (Hz):
    // * Possible bit rates with sample rates 11025, 12000, 22050, 24000 :
    //   8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160
    // * Possible bit rates with selected sample rates 44100, 48000 :
    //   32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320
    //........................................................................
    function set_Bitrate(dwBitrate: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Variable/Constant Bitrate Switch:
    // "0 = Constant Bitrate active"    "1 = Variable Bitrate active"
    //-------------------------------------------------------------------------
    // Get variable bitrate flag (1 or 0)
    function get_Variable(out dwVariable: DWORD): HRESULT; stdcall;
    // Set variable bitrate flag (1 or 0)
    function set_Variable(dwVariable: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Variable Bitrate Min Vaulue
    //-------------------------------------------------------------------------
    // Get variable bitrate in Kbits/s - Supported Values (Kbits/s):
    // 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 192, 224,
    // 256, 320
    function get_VariableMin(out dwmin: DWORD): HRESULT; stdcall;
    // Set variable bitrate in Kbits/s
    //........................................................................
    // Setting the correct supported bit rate (Kbits/s) values depends on the
    // selected sample rate (Hz):
    // * Possible bit rates with sample rates 11025, 12000, 22050, 24000 :
    //   8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160
    // * Possible bit rates with selected sample rates 44100, 48000 :
    //   32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320
    //........................................................................
    function set_VariableMin(dwmin: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Variable Bitrate Max Vaulue
    //-------------------------------------------------------------------------
    // Get variable bitrate in Kbits/s - Supported Values (Kbits/s):
    // 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 192, 224,
    // 256, 320
    function get_VariableMax(out dwmax: DWORD): HRESULT; stdcall;
    // Set variable bitrate in Kbits/s
    //........................................................................
    // Setting the correct supported bit rate (Kbits/s) values depends on the
    // selected sample rate (Hz):
    // * Possible bit rates with sample rates 11025, 12000, 22050, 24000 :
    //   8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160
    // * Possible bit rates with selected sample rates 44100, 48000 :
    //   32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320
    //........................................................................
    function set_VariableMax(dwmax: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Quality Supported Values
    // 2 = Best, 3 = High, 4 = High, 5 = Medium,
    // 6 = Medium, 7 = Low , 8 = Low, 9 = Fast Mode
    //-------------------------------------------------------------------------
    // Get compression quality
    function get_Quality(out dwQuality: DWORD): HRESULT; stdcall;
    // Set compression quality
    function set_Quality(dwQuality: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Variable Bitrate Quality Supported Values:
    //  0 = 0-~1:4, 1 = 1-~1:5, 2 = 2-~1:6, 3 = 3-~1:7, 4 = 4-~1:9,
    //  5 = 5-~1:9, 6= 6-~1:10, 7 = 7-~1:11, 8 = 8-~1:12, 9 = 9-~1:14
    //-------------------------------------------------------------------------
    // Get VBR quality
    function get_VariableQ(out dwVBRq: DWORD): HRESULT; stdcall;
    // Set VBR quality
    function set_VariableQ(dwVBRq: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Get source sample rate. Return E_FAIL if input pin
    // in not connected.
    function get_SourceSampleRate(out dwSampleRate: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Get source number of channels. Return E_FAIL if
    // input pin is not connected.
    function get_SourceChannels(out dwChannels: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Sample Rate Settings
    //-------------------------------------------------------------------------
    // Get Sample Rate for compressed audio bitstream
    //.........................................................................
    // "get_SampleRate" returns assigned sample rate. Possible return values
    //                  are: 48000, 24000, 12000, 44100, 22050, 11025
    function get_SampleRate(out dwSampleRate: DWORD): HRESULT; stdcall;
    // Set Sample Rate. (see genaudio spec for details)
    //.........................................................................
    // "set_SampleRate" possible set values depends on the input Sample rate.
    // * Input Sample Rates of 48000, 24000, 12000 allows to set the dest sample
    //   Rates: 48000, 24000, 12000
    // * Input Sample Rates of 44100, 22050, 11025 allows to set the dest sample
    //   Rates: 44100, 22050, 11025
    function set_SampleRate(dwSampleRate: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Channel mode settings
    //-------------------------------------------------------------------------
    // Supported values:
    // 0 = Standard stereo, 1 = Joint stereo, 2 = Dual Channel, 3 = Mono
    ////////////////////////////////////////////////////////////////////////////
    // Get channel mode. See genaudio.h for details
    function get_ChannelMode(out dwChannelMode: DWORD): HRESULT; stdcall;
    // Set channel mode
    function set_ChannelMode(dwChannelMode: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Enable/disable CRC:
    // "0 = CRC disbaled"    "1 = CRC enabled"
    //-------------------------------------------------------------------------
    // Is CRC enabled?
    function get_CRCFlag(out dwFlag: DWORD): HRESULT; stdcall;
    // Enable/disable CRC
    function set_CRCFlag(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Enable/disable force mono:
    // "0 = force mono disbaled"    "1 = force mono enabled"
    //-------------------------------------------------------------------------
    // get Force mono
    function get_ForceMono(out dwFlag: DWORD): HRESULT; stdcall;
    // set Force mono
    function set_ForceMono(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // duration settings
    //-------------------------------------------------------------------------
    function get_SetDuration(out dwFlag: DWORD): HRESULT; stdcall;
    function set_SetDuration(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'original' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_OriginalFlag(out dwFlag: DWORD): HRESULT; stdcall;
    function set_OriginalFlag(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'copyright' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_CopyrightFlag(out dwFlag: DWORD): HRESULT; stdcall;
    function set_CopyrightFlag(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Enforce VBR Minimum bitrate' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_EnforceVBRmin(out dwFlag: DWORD): HRESULT; stdcall;
    function set_EnforceVBRmin(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Voice' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_VoiceMode(out dwFlag: DWORD): HRESULT; stdcall;
    function set_VoiceMode(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Keep All Frequencies' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_KeepAllFreq(out dwFlag: DWORD): HRESULT; stdcall;
    function set_KeepAllFreq(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Strict ISO complience' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_StrictISO(out dwFlag: DWORD): HRESULT; stdcall;
    function set_StrictISO(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Disable short block' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_NoShortBlock(out dwDisable: DWORD): HRESULT; stdcall;
    function set_NoShortBlock(dwDisable: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Xing VBR Tag' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_XingTag(out dwXingTag: DWORD): HRESULT; stdcall;
    function set_XingTag(dwXingTag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'Forced mid/ side stereo' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    // NOTE: this Flag do only take effect if Channel mode settings Joint Stereo
    //       is selected:
    //-------------------------------------------------------------------------
    function get_ForceMS(out dwFlag: DWORD): HRESULT; stdcall;
    function set_ForceMS(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    // Control 'ModeFixed' flag - supported values: 1, 0
    //-------------------------------------------------------------------------
    function get_ModeFixed(out dwFlag: DWORD): HRESULT; stdcall;
    function set_ModeFixed(dwFlag: DWORD): HRESULT; stdcall;
    ////////////////////////////////////////////////////////////////////////////

    //Receive the block of encoder
    //configuration parametres
    function get_ParameterBlockSize(out pcBlock: Byte; out pdwSize: DWORD): HRESULT; stdcall;
    // Set encoder configuration parametres
    function set_ParameterBlockSize(pcBlock: PByte; dwSize: DWORD): HRESULT; stdcall;

    // Set default audio encoder parameters depending
    // on current input stream type
    function DefaultAudioEncoderProperties: HRESULT; stdcall;

    // By default the modified properties are not saved to
    // registry immediately, so the filter needs to be
    // forced to do this. Omitting this steps may lead to
    // misbehavior and confusing results.
    function LoadAudioEncoderPropertiesFromRegistry: HRESULT; stdcall;
    function SaveAudioEncoderPropertiesToRegistry: HRESULT; stdcall;

    // Determine, whether the filter can be configured. If this
    // functions returs E_FAIL, input format hasn't been
    // specified and filter behavior unpredicated. If S_OK,
    // the filter could be configured with correct values.
    function InputTypeDefined: HRESULT; stdcall;

    // Reconnects output pin (crucial for Fraunhofer MPEG Layer-3 Decoder)
    function ApplyChanges: HRESULT; stdcall;

    // Allow output sample overlap in terms of DirectShow
    // timestamps (i.e. when sample's start time is less
    // than previous sample's end time). Avi Mux doesn't like this
    // Control 'ModeFixed' flag
    function set_SampleOverlap(dwFlag: DWORD): HRESULT; stdcall;
    function get_SampleOverlap(out dwFlag: DWORD): HRESULT; stdcall;
    
  end;

implementation

end.

